@@ -1,32 +1,33 @@
 /******************************************************************************
+    AUTHOR:
+    File written and Copyrighted by Zachary Dovel. All Rights Reserved.
 
-  FILE:
-  src/main-interface-console.c
+    LICENSE:
+    This file is part of gWaei.
 
-  DESCRIPTION:
-  Used as a go between for functions interacting with the console.
-
-  AUTHOR:
-  File written and Copyrighted by Zachary Dovel. All Rights Reserved.
-
-  LICENSE:
-  This file is part of gWaei.
-
-  gWaei is free software: you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation, either version 3 of the License, or
-  (at your option) any later version.
-
-  gWaei is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-  
-  You should have received a copy of the GNU General Public License
-  along with gWaei.  If not, see <http://www.gnu.org/licenses/>.
+    gWaei is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
 
+    gWaei is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+    
+    You should have received a copy of the GNU General Public License
+    along with gWaei.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
 
+//!
+//! @file src/main-interface-console.c
+//!
+//! @brief Abstraction layer for the console
+//!
+//! Used as a go between for functions interacting with the console.
+//!
+
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -37,75 +38,52 @@
 
 #include <gwaei/definitions.h>
 #include <gwaei/regex.h>
-#include <gwaei/dictionaries.h>
-#include <gwaei/history.h>
+#include <gwaei/dictionary-objects.h>
+#include <gwaei/search-objects.h>
 #include <gwaei/engine.h>
 #include <gwaei/preferences.h>
 
 gboolean quiet_switch = FALSE;
 gboolean exact_switch = FALSE;
 
-void gwaei_console_uninstall_dictionary_by_name(char *name)
+void gw_console_uninstall_dictionary_by_name(char *name)
 {
-    DictionaryInfo* di;
-    di = dictionarylist_get_dictionary_by_name(name);
+    GwDictInfo* di;
+    di = gw_dictlist_get_dictionary_by_name (name);
+    if (di == NULL) return;
 
-    printf("*  ");
-    printf(gettext("Removing %s dictionary..."), name);
-    printf("\n");
+    gw_io_delete_dictionary_file(di);
+    di->status = GW_DICT_STATUS_NOT_INSTALLED;
+    di->load_position = -1;
 
-    printf("*   ");
-    printf(gettext("Removing %s..."), di->path);
-    printf("\n");
-    g_remove(di->path);
-
-    printf("*   ");
-    printf(gettext("Removing %s..."), di->gz_path);
-    printf("\n");
-    g_remove(di->gz_path);
-
-    printf("*   ");
-    printf(gettext("Removing %s..."), di->sync_path);
-    printf("\n");
-    g_remove(di->sync_path);
-
-    di->status = NOT_INSTALLED;
-
-    if (di->id == MIX)
-      gwaei_console_uninstall_dictionary_by_name (name);
-    if (di->id == NAMES &&
-        dictionarylist_dictionary_get_status_by_id (PLACES) == INSTALLED)
-      gwaei_console_uninstall_dictionary_by_name ("Places");
-    if (di->id == PLACES &&
-        dictionarylist_dictionary_get_status_by_id (NAMES) == INSTALLED)
-      gwaei_console_uninstall_dictionary_by_name ("Names");
-
-
-    di->status = NOT_INSTALLED;
+    if (di->id == GW_DICT_KANJI || di->id == GW_DICT_RADICALS)
+      gw_console_uninstall_dictionary_by_name ("Mix");
+    else if (di->id == GW_DICT_NAMES)
+      gw_console_uninstall_dictionary_by_name ("Places");
 }
 
 
-gboolean gwaei_console_install_dictionary_by_name(char *name)
+gboolean gw_console_install_dictionary_by_name(char *name)
 {
-    DictionaryInfo* di;
-    di = dictionarylist_get_dictionary_by_name(name);
+    GwDictInfo* di;
+    di = gw_dictlist_get_dictionary_by_alias(name);
 
     char *path = di->path;
     char *sync_path = di->sync_path;
     char *gz_path = di->gz_path;
 
-    if (di->status != NOT_INSTALLED || di->id == PLACES) return FALSE;
-    di->status = INSTALLING;
+    if (di->status != GW_DICT_STATUS_NOT_INSTALLED || di->id == GW_DICT_PLACES) return FALSE;
+    di->status = GW_DICT_STATUS_INSTALLING;
     
     char fallback_uri[100];
-    gwaei_util_strncpy_fallback_from_key (fallback_uri, di->gckey, 100);
+    gw_util_strncpy_fallback_from_key (fallback_uri, di->gckey, 100);
  
     char uri[100];
-    gwaei_pref_get_string (uri, di->gckey, fallback_uri, 100);
+    gw_pref_get_string (uri, di->gckey, fallback_uri, 100);
 
     //Make sure the download folder exits
     char download_path[FILENAME_MAX] = "";
-    get_waei_directory(download_path);
+    gw_util_get_waei_directory(download_path);
     strcat(download_path, G_DIR_SEPARATOR_S);
     strcat(download_path, "download");
 
@@ -118,36 +96,36 @@ gboolean gwaei_console_install_dictionary_by_name(char *name)
     if (ret && g_file_test (uri, G_FILE_TEST_IS_REGULAR))
     {
       printf("  %s\n", gettext("Coping file..."));
-      ret = gwaei_io_copy_dictionary_file (uri, gz_path, NULL);
+      ret = gw_io_copy_dictionary_file (uri, gz_path, NULL);
     }
     else if (ret)
     {
       printf("* %s\n", gettext("Downloading file..."));
-      ret = gwaei_io_download_dictionary_file (uri, gz_path, NULL, NULL);
+      ret = gw_io_download_dictionary_file (uri, gz_path, NULL, NULL);
     }
 
     if (ret)
     {
       printf("* %s\n", gettext("Gunzipping file..."));
-      ret = gwaei_io_gunzip_dictionary_file(gz_path, NULL);
+      ret = gw_io_gunzip_dictionary_file(gz_path, NULL);
     }
    
     if (ret)
     {
       printf("* %s\n",gettext("Converting encoding..."));
-      ret = gwaei_io_copy_with_encoding(sync_path, path, "EUC-JP","UTF-8");
+      ret = gw_io_copy_with_encoding(sync_path, path, "EUC-JP","UTF-8");
     }
 
     if (ret)
-      di->status = INSTALLED;
+      di->status = GW_DICT_STATUS_INSTALLED;
     else
-      di->status = NOT_INSTALLED;
+      di->status = GW_DICT_STATUS_NOT_INSTALLED;
 
     //Special dictionary post processing
     if (ret)
     {
       printf("* %s\n", gettext("Postprocessing..."));
-      dictionarylist_preform_postprocessing_by_name(name);
+      gw_dictlist_preform_postprocessing_by_name(name);
     }
 
     return ret;
@@ -156,7 +134,7 @@ gboolean gwaei_console_install_dictionary_by_name(char *name)
 
 static void print_about_program ()
 {
-    printf ("[1;31m%s Version %s[0m[1m with ", PACKAGE, VERSION);
+    printf ("[1;31m%s Version %s[0m with ", PACKAGE, VERSION);
     if (strcmp (INTERFACE, "NONE") == 0)
       printf ("no");
     else if (strcmp (INTERFACE, "gwaei"))
@@ -172,11 +150,11 @@ static void print_about_program ()
 
 static void print_search_start_banner(char *query, char *dictionary)
 {
-    printf("[1m");
+    printf("");
     printf("%s", gettext("Searching for \""));
-    printf("[1;31m%s[0m[1m", query);
+    printf("[1;31m%s[0m", query);
     printf("%s", gettext("\" in the "));
-    printf("[1;31m%s[0m[1m", dictionary);
+    printf("[1;31m%s[0m", dictionary);
     printf("%s", gettext(" dictionary..."));
     printf("[0m\n\n");
 
@@ -196,12 +174,12 @@ static void print_installable_dictionaries()
 
     int i = 0; 
 
-    DictionaryInfo* di;
-    GList *list = dictionarylist_get_list();
+    GwDictInfo* di;
+    GList *list = gw_dictlist_get_list();
     while (list != NULL)
     {
       di = list->data;
-      if (di->gckey[0] != '\0' && di->status == NOT_INSTALLED)
+      if (di->gckey[0] != '\0' && di->status == GW_DICT_STATUS_NOT_INSTALLED)
       {
         if (i != 0) printf(" ");
         printf("%s", di->name);
@@ -227,12 +205,12 @@ static void print_uninstallable_dictionaries()
 
     int i = 0; 
 
-    DictionaryInfo* di;
-    GList *list = dictionarylist_get_list();
+    GwDictInfo* di;
+    GList *list = gw_dictlist_get_list();
     while (list != NULL)
     {
       di = list->data;
-      if (di->status == INSTALLED)
+      if (di->status == GW_DICT_STATUS_INSTALLED)
       {
         if (i != 0) printf(" ");
         printf("%s", di->name);
@@ -259,12 +237,12 @@ static void print_available_dictionaries()
 
     int i = 0; 
 
-    DictionaryInfo* di;
-    GList *list = dictionarylist_get_list();
+    GwDictInfo* di;
+    GList *list = gw_dictlist_get_list();
     while (list != NULL)
     {
       di = list->data;
-      if (di->status == INSTALLED)
+      if (di->status == GW_DICT_STATUS_INSTALLED)
       {
         if (i != 0) printf(" ");
         printf("%s", di->name);
@@ -367,8 +345,8 @@ void initialize_console_interface(int argc, char **argv)
   int leftover;
   char query[MAX_QUERY];
 
-  DictionaryInfo *di;
-  di = dictionarylist_get_dictionary_by_name("English");
+  GwDictInfo *di;
+  di = gw_dictlist_get_dictionary_by_alias("English");
 
   char *args[argc];
   int total_args;
@@ -381,7 +359,6 @@ void initialize_console_interface(int argc, char **argv)
     if (is_switch (argv[i], "-e", "--exact"))
     {
       exact_switch = TRUE;
-      less_relevant_results_show = FALSE;
     }
 
     else if (is_switch (argv[i], "-q", "--quiet"))
@@ -392,7 +369,7 @@ void initialize_console_interface(int argc, char **argv)
     else if (is_switch (argv[i], "-d", "--dictionary"))
     {
       i++;
-      di = dictionarylist_get_dictionary_by_name(argv[i]);
+      di = gw_dictlist_get_dictionary_by_alias(argv[i]);
     }
 
     else
@@ -431,32 +408,32 @@ void initialize_console_interface(int argc, char **argv)
   //User wants to install dictionary
   else if (total_args == 2 && is_switch (args[0], "-i", "--install"))
   {
-    printf(gettext("%sTrying to install %s%s%s...%s"), "[1m", "[1;31m", args[1], "[0m[1m", "\n[0m");
+    printf(gettext("%sTrying to install %s%s%s...%s"), "", "[1;31m", args[1], "[0m", "\n[0m");
 
-    di = dictionarylist_get_dictionary_by_name(args[1]);
+    di = gw_dictlist_get_dictionary_by_alias(args[1]);
 
-    if (di != NULL && di->status != NOT_INSTALLED && di->gckey[0] != '\0')
+    if (di != NULL && di->status != GW_DICT_STATUS_NOT_INSTALLED && di->gckey[0] != '\0')
     {
-      //printf("[1m");
+      //printf("");
       printf("%s\n", gettext("Already Installed"));
       //printf("[0m");
     }
     else if (di == NULL || di->gckey[0] == '\0')
     {
-      //printf("[1m");
+      //printf("");
       printf("%s\n", gettext("That dictionary is not installable with this mechanism"));
       //printf("[0m");
       exit (1);
     }
-    else if (gwaei_console_install_dictionary_by_name (args[1]))
+    else if (gw_console_install_dictionary_by_name (args[1]))
     {
-      printf("[1m");
+      printf("");
       printf(gettext("Finished"));
       printf("[0m\n");
     }
     else
     {
-      printf("[1m");
+      printf("");
       printf(gettext("Failed"));
       printf("[0m\n");
     }
@@ -471,13 +448,13 @@ void initialize_console_interface(int argc, char **argv)
   //User wants to uninstall dictionary
   else if (total_args == 2 && is_switch (args[0], "-u", "--uninstall"))
   {
-    printf(gettext("%sTrying to uninstall %s%s%s...%s"), "[1m", "[1;31m", args[1], "[0m[1m", "\n[0m");
+    printf(gettext("%sTrying to uninstall %s%s%s...%s"), "", "[1;31m", args[1], "[0m", "\n[0m");
 
-    if (dictionarylist_check_if_loaded_by_name(argv[2]))
+    if (gw_dictlist_check_if_loaded_by_name(argv[2]))
     {
-      gwaei_console_uninstall_dictionary_by_name(argv[2]);
+      gw_console_uninstall_dictionary_by_name(argv[2]);
 
-      printf("[1m");
+      printf("");
       printf(gettext("Finished"));
       printf("[0m\n");
     }
@@ -491,28 +468,28 @@ void initialize_console_interface(int argc, char **argv)
   //User wants to sync dictionaries
   else if (total_args == 1 && is_switch (args[0], "-s", "--sync"))
   {
-    printf("[1m");
+    printf("");
     printf(gettext("Syncing possible installed dictionaries..."));
     printf("[0m\n");
 
-    DictionaryInfo* di;
-    GList *list = dictionarylist_get_list();
+    GwDictInfo* di;
+    GList *list = gw_dictlist_get_list();
     while (list != NULL && error == NULL)
     {
       di = list->data;
-      dictionarylist_sync_dictionary (di, &error);
+      gw_dictlist_sync_dictionary (di, &error);
       list = list->next;
     }
 
     if (error == NULL)
     {
-      printf("[1m");
+      printf("");
       printf(gettext("Finished"));
       printf("[0m\n");
     }
     else
     {
-      printf("[1m");
+      printf("");
       printf("%s", error->message);
       printf("[0m\n");
       g_error_free (error);
@@ -522,7 +499,7 @@ void initialize_console_interface(int argc, char **argv)
 
 
   //Make sure the selected dictionary exists
-  else if (di == NULL || di->status != INSTALLED)
+  else if (di == NULL || di->status != GW_DICT_STATUS_INSTALLED)
   {
     printf(gettext("Requested dictionary not found!\n"));
   }
@@ -551,21 +528,24 @@ void initialize_console_interface(int argc, char **argv)
     if (quiet_switch == FALSE)
       print_search_start_banner(query, di->name);
 
-    char preformatted_query[MAX_QUERY];
+    GwSearchItem *item;
+    item = gw_searchitem_new(query, di, GW_TARGET_CONSOLE);
+    if (item == NULL){
+    	//TODO: Use GError instead
+    	printf("Out of memory. Exiting.");
+    	exit (EXIT_FAILURE);
+    }
 
-    strcpy_with_query_preformatting (preformatted_query, query,
-                                     di, GWAEI_TARGET_CONSOLE);
+    item->show_less_relevant_results = !exact_switch;
 
-    SearchItem *item;
-    item = searchitem_new(preformatted_query, di, GWAEI_TARGET_CONSOLE);
     if (item != NULL )
     {
-      gwaei_search_get_results (item);
+      gw_search_get_results (item);
 
       if (quiet_switch == FALSE)
       {
-        printf("[1m");
-        printf("%s", gettext("Found "));
+        printf("");
+        printf("\n%s", gettext("Found "));
         printf("%d", item->total_results);
         printf("%s", gettext(" Results"));
 
@@ -588,3 +568,101 @@ void initialize_console_interface(int argc, char **argv)
 }
 
 
+void gw_console_append_normal_results (GwSearchItem *item, gboolean unused)
+{
+    GwResultLine *resultline = item->resultline;
+    //Kanji
+    printf("[32m%s", resultline->kanji_start);
+    //Furigana
+    if (resultline->furigana_start)
+    {
+      printf(" [%s]", resultline->furigana_start);
+    }
+    //Other info
+    if (resultline->classification_start)
+    {
+      printf("[0m %s", resultline->classification_start);
+    }
+    if (resultline->important)
+    {
+      printf("[0m %s", "P");
+    }
+    printf("\n");
+    //Definitions
+    int i = 0;
+    while (i < resultline->def_total)
+    {
+      printf("[0m      [35m%s [0m%s\n", resultline->number[i], resultline->def_start[i]);
+      i++;
+    }
+    printf("\n");
+
+}
+
+void gw_console_append_kanji_results (GwSearchItem *item, gboolean unused)
+{
+    GwResultLine *resultline = item->resultline;
+    //Kanji
+    printf("[32;1m%s[0m\n", resultline->kanji);
+    if (resultline->radicals) printf("%s%s\n", gettext("[35mRadicals:[0m"), resultline->radicals);
+
+    char line_started = FALSE;
+    if (resultline->strokes)
+    {
+      line_started = TRUE;
+      printf("%s%s", gettext("[35mStroke:[0m"), resultline->strokes);
+    }
+    if (resultline->frequency)
+    {
+      if (line_started) printf(" ");
+      line_started = TRUE;
+      printf("%s%s", gettext("[35mFreq:[0m"), resultline->frequency);
+    }
+    if (resultline->grade)
+    {
+      if (line_started) printf(" ");
+      line_started = TRUE;
+      printf("%s%s", gettext("[35mGrade:[0m"), resultline->grade);
+    }
+    if (resultline->jlpt)
+    {
+      if (line_started) printf(" ");
+      line_started = TRUE;
+      printf("%s%s", gettext("[35mJLPT:[0m"), resultline->jlpt);
+    }
+    if (line_started) printf("\n");
+    if (resultline->readings[0]) printf("%s%s", gettext("[35mReadings:[0m"), resultline->readings[0]);
+    if (resultline->readings[1]) printf("%s", resultline->readings[1]);
+    printf("\n");
+
+    if (resultline->meanings) printf("%s%s\n", gettext("[35mMeanings:[0m"), resultline->meanings);
+    printf("\n");
+
+}
+
+void gw_console_append_radical_results (GwSearchItem *item, gboolean unused)
+{
+    GwResultLine *resultline = item->resultline;
+    printf("[32;1m%s:[0m %s\n\n", resultline->kanji, resultline->radicals);
+}
+
+void gw_console_append_examples_results (GwSearchItem *item, gboolean unused)
+{
+    GwResultLine *resultline = item->resultline;
+    int i = 0;
+    while (resultline->number[i] != NULL && resultline->def_start[i] != NULL)
+    {
+      if (resultline->number[i][0] == 'A' || resultline->number[i][0] == 'B')
+        printf("[32;1m%s:[0m\t%s\n", resultline->number[i], resultline->def_start[i]);
+      else
+        printf("\t%s\n", resultline->def_start[i]);
+      i++;
+    }
+    printf("\n");
+}
+
+void gw_console_append_unknown_results (GwSearchItem *item, gboolean unused)
+{
+    GwResultLine *resultline = item->resultline;
+    printf("%s\n", resultline->string);
+}
