@@ -1,32 +1,34 @@
 /******************************************************************************
+    AUTHOR:
+    File written and Copyrighted by Zachary Dovel. All Rights Reserved.
 
-  FILE:
-  src/preferences-gconf.c
+    LICENSE:
+    This file is part of gWaei.
 
-  DESCRIPTION:
-  Calls to gconf are abstracted out here. This is the gconf version.
-
-  AUTHOR:
-  File written and Copyrighted by Zachary Dovel. All Rights Reserved.
-
-  LICENSE:
-  This file is part of gWaei.
-
-  gWaei is free software: you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation, either version 3 of the License, or
-  (at your option) any later version.
-
-  gWaei is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-  
-  You should have received a copy of the GNU General Public License
-  along with gWaei.  If not, see <http://www.gnu.org/licenses/>.
+    gWaei is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
 
+    gWaei is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+    
+    You should have received a copy of the GNU General Public License
+    along with gWaei.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
 
+//!
+//! @file src/preferences-gconf.c
+//!
+//! @brief Abstraction layer for gconf preferences
+//!
+//! Allows access to gconf with the ability to specify backup preferences upon
+//! failure to get the preference value.
+//!
+
+
 #include <string.h>
 #include <regex.h>
 #include <locale.h>
@@ -37,7 +39,7 @@
 #include <gwaei/regex.h>
 
 
-int gwaei_pref_get_int (char *key, int backup)
+int gw_pref_get_int (char *key, int backup)
 {
     GConfClient *client;
     client = gconf_client_get_default ();
@@ -60,7 +62,7 @@ int gwaei_pref_get_int (char *key, int backup)
 }
 
 
-int gwaei_pref_get_default_int (char *key, int backup)
+int gw_pref_get_default_int (char *key, int backup)
 {
     GConfClient *client;
     client = gconf_client_get_default ();
@@ -70,12 +72,16 @@ int gwaei_pref_get_default_int (char *key, int backup)
     GError *err = NULL;
 
     value = gconf_client_get_default_from_schema (client, key, &err);
-    if (err != NULL || (value != NULL && value->type != GCONF_VALUE_INT))
+    if (err != NULL)
     {
       g_error_free (err);
       err = NULL;
       return_value = backup;
     }
+    if (value == NULL || value->type != GCONF_VALUE_INT)
+    {
+      return_value = backup;
+    }
     else
     {
       return_value = gconf_value_get_int (value);
@@ -86,7 +92,7 @@ int gwaei_pref_get_default_int (char *key, int backup)
 }
 
 
-void gwaei_pref_set_int (char *key, int request)
+void gw_pref_set_int (char *key, int request)
 {
     GConfClient *client;
     client = gconf_client_get_default ();
@@ -95,7 +101,7 @@ void gwaei_pref_set_int (char *key, int request)
 }
 
 
-gboolean gwaei_pref_get_boolean (char *key, gboolean backup)
+gboolean gw_pref_get_boolean (char *key, gboolean backup)
 {
     GConfClient *client;
     client = gconf_client_get_default ();
@@ -118,7 +124,7 @@ gboolean gwaei_pref_get_boolean (char *key, gboolean backup)
 }
 
 
-gboolean gwaei_pref_get_default_boolean (char *key, gboolean backup)
+gboolean gw_pref_get_default_boolean (char *key, gboolean backup)
 {
     GConfClient *client;
     client = gconf_client_get_default ();
@@ -128,12 +134,16 @@ gboolean gwaei_pref_get_default_boolean (char *key, gboolean backup)
     GError *err = NULL;
 
     value = gconf_client_get_default_from_schema (client, key, &err);
-    if (err != NULL || (value != NULL && value->type != GCONF_VALUE_BOOL))
+    if (err != NULL)
     {
       g_error_free (err);
       err = NULL;
       return_value = backup;
     }
+    else if (value == NULL || value->type != GCONF_VALUE_BOOL)
+    {
+      return_value = backup;
+    }
     else
     {
       return_value = gconf_value_get_bool (value);
@@ -144,7 +154,7 @@ gboolean gwaei_pref_get_default_boolean (char *key, gboolean backup)
 }
 
 
-void gwaei_pref_set_boolean (char *key, gboolean request)
+void gw_pref_set_boolean (char *key, gboolean request)
 {
     GConfClient *client;
     client = gconf_client_get_default ();
@@ -153,7 +163,7 @@ void gwaei_pref_set_boolean (char *key, gboolean request)
 }
 
 
-char* gwaei_pref_get_string (char *output, char *key, char* backup, int n)
+char* gw_pref_get_string (char *output, char *key, char* backup, int n)
 {
     GConfClient *client;
     client = gconf_client_get_default ();
@@ -170,6 +180,10 @@ char* gwaei_pref_get_string (char *output, char *key, char* backup, int n)
       return_value = NULL;
       strncpy(output, backup, n);
     }
+    else if (return_value == NULL || strlen(return_value) == 0)
+    {
+      strncpy(output, backup, n);
+    }
     else
     {
       strncpy(output, return_value, n);
@@ -180,8 +194,7 @@ char* gwaei_pref_get_string (char *output, char *key, char* backup, int n)
     return return_value;
 }
 
-
-const char* gwaei_pref_get_default_string (char *key, char* backup)
+const char* gw_pref_get_default_string (char *key, char* backup)
 {
     GConfClient *client;
     client = gconf_client_get_default ();
@@ -206,12 +219,13 @@ const char* gwaei_pref_get_default_string (char *key, char* backup)
     return return_value;
 }
 
-const char* gwaei_pref_free_string (char *sting)
+
+const char* gw_pref_free_string (char *sting)
 {
     
 }
 
-void gwaei_pref_set_string (char *key, char* request)
+void gw_pref_set_string (char *key, const char* request)
 {
     GConfClient *client;
     client = gconf_client_get_default ();
@@ -241,7 +255,7 @@ void do_dictionary_source_gconf_key_changed_action (GConfClient* client,
         key_ptr--;
       key_ptr++;
 
-      gwaei_ui_set_dictionary_source(key_ptr, gconf_value_get_string(value));
+      gw_ui_set_dictionary_source(key_ptr, gconf_value_get_string(value));
     }
 }
 
@@ -255,9 +269,9 @@ void do_toolbar_style_pref_changed_action( GConfClient* client,
     value = gconf_entry_get_value(entry);
 
     if (value != NULL && value->type == GCONF_VALUE_STRING)
-      gwaei_ui_set_toolbar_style(gconf_value_get_string(value));
+      gw_ui_set_toolbar_style(gconf_value_get_string(value));
     else
-      gwaei_ui_set_toolbar_style("both");
+      gw_ui_set_toolbar_style("both");
 }
 
 
@@ -270,9 +284,9 @@ void do_toolbar_show_pref_changed_action ( GConfClient* client,
     value = gconf_entry_get_value(entry);
 
     if (value != NULL && value->type == GCONF_VALUE_BOOL)
-      gwaei_ui_set_toolbar_show(gconf_value_get_bool(value));
+      gw_ui_set_toolbar_show(gconf_value_get_bool(value));
     else
-      gwaei_ui_set_toolbar_show(FALSE);
+      gw_ui_set_toolbar_show(FALSE);
 }
 
 
@@ -283,27 +297,27 @@ void do_font_size_pref_changed_action ( GConfClient* client,
 {
     //Get the size from the GCONF key
     int size;
-    size = gconf_client_get_int ( client, GCKEY_GWAEI_FONT_SIZE, NULL);
+    size = gconf_client_get_int ( client, GCKEY_GW_FONT_SIZE, NULL);
 
     //If the value is strange, get the default value
     if (size < MIN_FONT_SIZE | size > MAX_FONT_SIZE)
     {
       GConfValue *value;
       value = gconf_client_get_default_from_schema ( client,
-                                                     GCKEY_GWAEI_FONT_SIZE,
+                                                     GCKEY_GW_FONT_SIZE,
                                                      NULL                   );
       size = gconf_value_get_int (value);
       if (value != NULL && size >= MIN_FONT_SIZE && size <= MAX_FONT_SIZE) {
-        gconf_client_set_int ( client, GCKEY_GWAEI_FONT_SIZE, size, NULL);
+        gconf_client_set_int ( client, GCKEY_GW_FONT_SIZE, size, NULL);
       }
       else
-        gconf_client_set_int ( client, GCKEY_GWAEI_FONT_SIZE, 12, NULL);
+        gconf_client_set_int ( client, GCKEY_GW_FONT_SIZE, 12, NULL);
       return;
     }
 
-    gwaei_ui_set_font("Sans", size);
+    gw_ui_set_font("Sans", size);
 
-    update_toolbar_buttons ();
+    gw_ui_update_toolbar_buttons ();
 }
 
 
@@ -316,9 +330,9 @@ void do_less_relevant_show_pref_changed_action ( GConfClient* client,
     value = gconf_entry_get_value(entry);
 
     if (value != NULL && value->type == GCONF_VALUE_BOOL)
-      gwaei_ui_set_less_relevant_show(gconf_value_get_bool(value));
+      gw_ui_set_less_relevant_show(gconf_value_get_bool(value));
     else
-      gwaei_ui_set_less_relevant_show(TRUE);
+      gw_ui_set_less_relevant_show(TRUE);
 }
 
 
@@ -336,9 +350,9 @@ void do_roman_kana_conv_pref_changed_action( GConfClient* client,
        selection = gconf_value_get_int(value);
 
        if (selection <= 2 && selection >= 0)
-         gwaei_ui_set_romanji_kana_conv(selection);
+         gw_ui_set_romaji_kana_conv(selection);
        else
-         gwaei_ui_set_romanji_kana_conv(2);
+         gw_ui_set_romaji_kana_conv(2);
     }
 }
 
@@ -352,9 +366,9 @@ void do_hira_kata_conv_pref_changed_action( GConfClient* client,
     value = gconf_entry_get_value(entry);
 
     if (value != NULL && value->type == GCONF_VALUE_BOOL)
-      gwaei_ui_set_hiragana_katakana_conv(gconf_value_get_bool(value));
+      gw_ui_set_hiragana_katakana_conv(gconf_value_get_bool(value));
     else
-      gwaei_ui_set_hiragana_katakana_conv(TRUE);
+      gw_ui_set_hiragana_katakana_conv(TRUE);
 }
 
 
@@ -367,9 +381,9 @@ void do_kata_hira_conv_pref_changed_action( GConfClient *client,
     value = gconf_entry_get_value(entry);
 
     if (value != NULL && value->type == GCONF_VALUE_BOOL)
-      gwaei_ui_set_katakana_hiragana_conv(gconf_value_get_bool(value));
+      gw_ui_set_katakana_hiragana_conv(gconf_value_get_bool(value));
     else
-      gwaei_ui_set_katakana_hiragana_conv(TRUE);
+      gw_ui_set_katakana_hiragana_conv(TRUE);
 }
 
 
@@ -382,14 +396,12 @@ void do_spellcheck_pref_changed_action( GConfClient* client,
     value = gconf_entry_get_value(entry);
 
     if (value != NULL && value->type == GCONF_VALUE_BOOL)
-      gwaei_ui_set_spellcheck(gconf_value_get_bool(value));
+      gw_sexy_ui_set_spellcheck(gconf_value_get_bool(value));
     else
-      gwaei_ui_set_spellcheck(TRUE);
+      gw_sexy_ui_set_spellcheck(TRUE);
 }
 
 
-#define IS_HEXCOLOR(color) (regexec(&re_hexcolor, (color), 1, NULL, 0) == 0)
-
 void do_color_value_changed_action( GConfClient* client, 
                                     guint cnxn_id,
                                     GConfEntry *entry,
@@ -406,7 +418,7 @@ void do_color_value_changed_action( GConfClient* client,
       if (regexec(&re_hexcolor, gconf_value_get_string(value), 1, NULL, 0) != 0)
       {
         value = gconf_client_get_default_from_schema (client,
-                                                      GCKEY_GWAEI_FONT_SIZE,
+                                                      GCKEY_GW_FONT_SIZE,
                                                       NULL                  );
         if (value != NULL && value->type == GCONF_VALUE_STRING)
           hex_string = gconf_value_get_string (value);
@@ -414,9 +426,9 @@ void do_color_value_changed_action( GConfClient* client,
           return;
       }
 
-      guint red   = gwaei_2digithexstrtoint(hex_string[1], hex_string[2]);
-      guint green = gwaei_2digithexstrtoint(hex_string[3], hex_string[4]);
-      guint blue  = gwaei_2digithexstrtoint(hex_string[5], hex_string[6]);
+      guint red   = gw_util_2digithexstrtoint(hex_string[1], hex_string[2]);
+      guint green = gw_util_2digithexstrtoint(hex_string[3], hex_string[4]);
+      guint blue  = gw_util_2digithexstrtoint(hex_string[5], hex_string[6]);
 
       //The last portion of the key happens to be the widget id
       const char *key = gconf_entry_get_key(entry);
@@ -425,13 +437,20 @@ void do_color_value_changed_action( GConfClient* client,
         key_ptr--;
       key_ptr++;
 
-      gwaei_ui_set_color_to_swatch(key_ptr, red, green, blue);
+      gw_ui_set_color_to_swatch(key_ptr, red, green, blue);
 
-      gwaei_reload_tagtable_tags();
+      gw_ui_reload_tagtable_tags();
     }
 }
 
 
+void do_update_dictionary_order_list_changed_action ()
+{
+      gw_ui_update_dictionary_orders ();
+}
+
+
+
 
 //
 //Preference initializations
@@ -439,7 +458,7 @@ void do_color_value_changed_action( GConfClient* client,
 
 
 
-void gwaei_prefs_initialize_preferences()
+void gw_prefs_initialize_preferences()
 {
   g_type_init();
 
@@ -452,12 +471,12 @@ void gwaei_prefs_initialize_preferences()
     gconf_client_add_dir   ( client, GCPATH_INTERFACE, 
                              GCONF_CLIENT_PRELOAD_NONE,
                              NULL                             );
-    gconf_client_add_dir   ( client, GCPATH_GWAEI, 
+    gconf_client_add_dir   ( client, GCPATH_GW, 
                              GCONF_CLIENT_PRELOAD_NONE,
                              NULL                            );
 
     //Add preference change notifiers
-    gconf_client_notify_add (client, GCKEY_GWAEI_LESS_RELEVANT_SHOW, 
+    gconf_client_notify_add (client, GCKEY_GW_LESS_RELEVANT_SHOW, 
                              do_less_relevant_show_pref_changed_action,
                              NULL, NULL, NULL                );
 
@@ -465,90 +484,120 @@ void gwaei_prefs_initialize_preferences()
                              do_toolbar_style_pref_changed_action,
                              NULL, NULL, NULL                 );
 
-    gconf_client_notify_add (client, GCKEY_GWAEI_TOOLBAR_SHOW, 
+    gconf_client_notify_add (client, GCKEY_GW_TOOLBAR_SHOW, 
                              do_toolbar_show_pref_changed_action,
                              NULL, NULL, NULL                );
 
-    gconf_client_notify_add (client, GCKEY_GWAEI_FONT_SIZE, 
+    gconf_client_notify_add (client, GCKEY_GW_FONT_SIZE, 
                              do_font_size_pref_changed_action,
                              NULL, NULL, NULL               );
 
-    gconf_client_notify_add (client, GCKEY_GWAEI_ROMAN_KANA, 
+    gconf_client_notify_add (client, GCKEY_GW_ROMAN_KANA, 
                              do_roman_kana_conv_pref_changed_action,
                              NULL, NULL, NULL                );
 
-    gconf_client_notify_add (client, GCKEY_GWAEI_HIRA_KATA, 
+    gconf_client_notify_add (client, GCKEY_GW_HIRA_KATA, 
                              do_hira_kata_conv_pref_changed_action,
                              NULL, NULL, NULL                );
 
-    gconf_client_notify_add (client, GCKEY_GWAEI_KATA_HIRA, 
+    gconf_client_notify_add (client, GCKEY_GW_KATA_HIRA, 
                              do_kata_hira_conv_pref_changed_action,
                              NULL, NULL, NULL                );
 
-    gconf_client_notify_add (client, GCKEY_GWAEI_SPELLCHECK, 
+    gconf_client_notify_add (client, GCKEY_GW_SPELLCHECK, 
                              do_spellcheck_pref_changed_action,
                              NULL, NULL, NULL                );
 
-    gconf_client_notify_add (client, GCKEY_GWAEI_MATCH_FG, 
+    gconf_client_notify_add (client, GCKEY_GW_MATCH_FG, 
                              do_color_value_changed_action,
                              NULL, NULL, NULL                );
 
-    gconf_client_notify_add (client, GCKEY_GWAEI_MATCH_BG, 
+    gconf_client_notify_add (client, GCKEY_GW_MATCH_BG, 
                              do_color_value_changed_action,
                              NULL, NULL, NULL                );
 
-    gconf_client_notify_add (client, GCKEY_GWAEI_HEADER_FG, 
+    gconf_client_notify_add (client, GCKEY_GW_HEADER_FG, 
                              do_color_value_changed_action,
                              NULL, NULL, NULL                );
 
-    gconf_client_notify_add (client, GCKEY_GWAEI_HEADER_BG, 
+    gconf_client_notify_add (client, GCKEY_GW_HEADER_BG, 
                              do_color_value_changed_action,
                              NULL, NULL, NULL                );
 
-    gconf_client_notify_add (client, GCKEY_GWAEI_COMMENT_FG, 
+    gconf_client_notify_add (client, GCKEY_GW_COMMENT_FG, 
                              do_color_value_changed_action,
                              NULL, NULL, NULL                );
 
     gconf_client_notify_add (client,
-                             GCKEY_GWAEI_ENGLISH_SOURCE, 
+                             GCKEY_GW_ENGLISH_SOURCE, 
                              do_dictionary_source_gconf_key_changed_action,
                              NULL, NULL, NULL                 );
 
     gconf_client_notify_add (client,
-                             GCKEY_GWAEI_KANJI_SOURCE, 
+                             GCKEY_GW_KANJI_SOURCE, 
                              do_dictionary_source_gconf_key_changed_action,
                              NULL, NULL, NULL                 );
 
     gconf_client_notify_add (client,
-                             GCKEY_GWAEI_NAMES_SOURCE, 
+                             GCKEY_GW_NAMES_SOURCE, 
                              do_dictionary_source_gconf_key_changed_action,
                              NULL, NULL, NULL                 );
 
     gconf_client_notify_add (client,
-                             GCKEY_GWAEI_RADICALS_SOURCE, 
+                             GCKEY_GW_RADICALS_SOURCE, 
                              do_dictionary_source_gconf_key_changed_action,
                              NULL, NULL, NULL                 );
 
+    gconf_client_notify_add (client,
+                             GCKEY_GW_EXAMPLES_SOURCE, 
+                             do_dictionary_source_gconf_key_changed_action,
+                             NULL, NULL, NULL                 );
+
+    gconf_client_notify_add (client,
+                             GCKEY_GW_FRENCH_SOURCE, 
+                             do_dictionary_source_gconf_key_changed_action,
+                             NULL, NULL, NULL                 );
+
+    gconf_client_notify_add (client,
+                             GCKEY_GW_GERMAN_SOURCE, 
+                             do_dictionary_source_gconf_key_changed_action,
+                             NULL, NULL, NULL                 );
+
+    gconf_client_notify_add (client,
+                             GCKEY_GW_SPANISH_SOURCE, 
+                             do_dictionary_source_gconf_key_changed_action,
+                             NULL, NULL, NULL                 );
+
+    gconf_client_notify_add (client,
+                             GCKEY_GW_LOAD_ORDER, 
+                             do_update_dictionary_order_list_changed_action,
+                             NULL, NULL, NULL                 );
+
 
     //Do an initial trigger of the notifications to set an initial state
-    gconf_client_notify (client, GCKEY_GWAEI_TOOLBAR_SHOW);
+    gconf_client_notify (client, GCKEY_GW_TOOLBAR_SHOW);
     gconf_client_notify (client, GCKEY_TOOLBAR_STYLE);
-    gconf_client_notify (client, GCKEY_GWAEI_FONT_SIZE);
-    gconf_client_notify (client, GCKEY_GWAEI_KATA_HIRA);
-    gconf_client_notify (client, GCKEY_GWAEI_HIRA_KATA);
-    gconf_client_notify (client, GCKEY_GWAEI_ROMAN_KANA);
-    gconf_client_notify (client, GCKEY_GWAEI_SPELLCHECK);
-    gconf_client_notify (client, GCKEY_GWAEI_LESS_RELEVANT_SHOW);
-    gconf_client_notify (client, GCKEY_GWAEI_MATCH_FG);
-    gconf_client_notify (client, GCKEY_GWAEI_MATCH_BG);
-    gconf_client_notify (client, GCKEY_GWAEI_HEADER_FG);
-    gconf_client_notify (client, GCKEY_GWAEI_HEADER_BG);
-    gconf_client_notify (client, GCKEY_GWAEI_COMMENT_FG);
-    gconf_client_notify (client, GCKEY_GWAEI_ENGLISH_SOURCE);
-    gconf_client_notify (client, GCKEY_GWAEI_KANJI_SOURCE);
-    gconf_client_notify (client, GCKEY_GWAEI_NAMES_SOURCE);
-    gconf_client_notify (client, GCKEY_GWAEI_PLACES_SOURCE);
-    gconf_client_notify (client, GCKEY_GWAEI_RADICALS_SOURCE);
+    gconf_client_notify (client, GCKEY_GW_FONT_SIZE);
+    gconf_client_notify (client, GCKEY_GW_KATA_HIRA);
+    gconf_client_notify (client, GCKEY_GW_HIRA_KATA);
+    gconf_client_notify (client, GCKEY_GW_ROMAN_KANA);
+    gconf_client_notify (client, GCKEY_GW_SPELLCHECK);
+    gconf_client_notify (client, GCKEY_GW_LESS_RELEVANT_SHOW);
+    gconf_client_notify (client, GCKEY_GW_MATCH_FG);
+    gconf_client_notify (client, GCKEY_GW_MATCH_BG);
+    gconf_client_notify (client, GCKEY_GW_HEADER_FG);
+    gconf_client_notify (client, GCKEY_GW_HEADER_BG);
+    gconf_client_notify (client, GCKEY_GW_COMMENT_FG);
+    gconf_client_notify (client, GCKEY_GW_ENGLISH_SOURCE);
+    gconf_client_notify (client, GCKEY_GW_KANJI_SOURCE);
+    gconf_client_notify (client, GCKEY_GW_NAMES_SOURCE);
+    gconf_client_notify (client, GCKEY_GW_PLACES_SOURCE);
+    gconf_client_notify (client, GCKEY_GW_RADICALS_SOURCE);
+    gconf_client_notify (client, GCKEY_GW_EXAMPLES_SOURCE);
+    gconf_client_notify (client, GCKEY_GW_FRENCH_SOURCE);
+    gconf_client_notify (client, GCKEY_GW_GERMAN_SOURCE);
+    gconf_client_notify (client, GCKEY_GW_SPANISH_SOURCE);
+    gconf_client_notify (client, GCKEY_GW_LOAD_ORDER);
 
   g_object_unref(client);
 }
