@@ -1,31 +1,33 @@
 /******************************************************************************
+    AUTHOR:
+    File written and Copyrighted by Zachary Dovel. All Rights Reserved.
 
-  FILE:
-  src/utilities.c
+    LICENSE:
+    This file is part of gWaei.
 
-  DESCRIPTION:
-  Holds some basic functions that are extremely handy for gWaei.
-
-  AUTHOR:
-  File written and Copyrighted by Zachary Dovel. All Rights Reserved.
-
-  LICENSE:
-  This file is part of gWaei.
-
-  gWaei is free software: you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation, either version 3 of the License, or
-  (at your option) any later version.
-
-  gWaei is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-  
-  You should have received a copy of the GNU General Public License
-  along with gWaei.  If not, see <http://www.gnu.org/licenses/>.
+    gWaei is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
 
+    gWaei is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+    
+    You should have received a copy of the GNU General Public License
+    along with gWaei.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
+
+//!
+//! @file src/utilities.c
+//!
+//! @brief Generic utility functions
+//!
+//! Holds some basic functions that are extremely handy for gWaei.
+//!
+
+
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -36,58 +38,92 @@
 #include <gwaei/definitions.h>
 #include <gwaei/utilities.h>
 
-int run_mode;
+static int run_mode;
 
-void gwaei_util_initialize_runmode(char* call)
+
+//!
+//! @brief Sets the runmode of the program
+//!
+//! The runmode describes tells what interface the program will be using to
+//! primarily interact with the user.  The two possible choices are GTK or
+//! the console.  QT is currently a red herring.
+//!
+//! @param call String describing the runmode to set.
+//!
+void gw_util_initialize_runmode (char* call)
 {
    char *call_ptr = &call[strlen(call)];
-   while (call_ptr != call && !G_IS_DIR_SEPARATOR(*call_ptr))
+   while (call_ptr != call && !G_IS_DIR_SEPARATOR (*call_ptr))
      call_ptr--;
-   if (G_IS_DIR_SEPARATOR(*call_ptr))
+   if (G_IS_DIR_SEPARATOR (*call_ptr))
      call_ptr++;
 
-   if (strcmp(call_ptr, "waei") == 0 || strcmp(INTERFACE, "NONE") == 0)
-     run_mode = GWAEI_CONSOLE_RUNMODE;
-   else if (strcmp(call_ptr, "gwaei") == 0)
-     run_mode = GWAEI_GTK_RUNMODE;
-   else if (strcmp(call_ptr, "kwaei") == 0)
-     run_mode = GWAEI_QT_RUNMODE;
+   if (strcmp (call_ptr, "waei") == 0 || strcmp (INTERFACE, "NONE") == 0)
+     run_mode = GW_CONSOLE_RUNMODE;
+   else if (strcmp (call_ptr, "gwaei") == 0)
+     run_mode = GW_GTK_RUNMODE;
+   else if (strcmp (call_ptr, "kwaei") == 0)
+     run_mode = GW_QT_RUNMODE;
 }
 
-int gwaei_util_get_runmode()
+
+//!
+//! @brief Gets the runmode of the program
+//!
+//! The runmode describes tells what interface the program will be using to
+//! primarily interact with the user.  The two possible outputs:
+//! GW_CONSOLE_RUNMODE or GW_GTK_RUNMODE.  QT is currently a red herring.
+//!
+//! @return Returns the current runmode as an integer
+//!
+int gw_util_get_runmode ()
 {
     return run_mode;
 }
 
 
-//A fairly fast int to acsii char funciton
-gboolean gwaei_itoa(int input, char *output, const int MAX)
+//!
+//! @brief Internal integer to string converson algorithm
+//!
+//! @param the input integer
+//! @param a preallocated char array to store the string
+//! @param the length of the array for the radix
+//! @return Returns true if there were no problems
+//!
+gboolean gw_util_itoa (int input, char *output, const int MAX)
 {
-  //digit_ptr is used to write and track the number
-  char buffer[MAX];
-  char *digit_ptr = &buffer[MAX - 1];
+    //digit_ptr is used to write and track the number
+    char buffer[MAX];
+    char *digit_ptr = &buffer[MAX - 1];
 
-  *digit_ptr = '\0';
-  //Write out the digits starting at the end of of the buffer
-  do
-  {
-    digit_ptr--;
-    *digit_ptr = input % 10;
-    *digit_ptr += 48;
-    input /= 10;
-  } while (input != 0 && digit_ptr != &output[0]);
+    *digit_ptr = '\0';
+    //Write out the digits starting at the end of of the buffer
+    do
+    {
+      digit_ptr--;
+      *digit_ptr = input % 10;
+      *digit_ptr += 48;
+      input /= 10;
+    } while (input != 0 && digit_ptr != &output[0]);
 
-  //Shift the digits to the beginning of the string
-  strcpy(output, digit_ptr);
+    //Shift the digits to the beginning of the string
+    strcpy(output, digit_ptr);
 
-  //Send return status
-  if (input != 0 && digit_ptr == &output[0])
-    return FALSE;
-  else
-    return TRUE;
+    //Send return status
+    if (input != 0 && digit_ptr == &output[0])
+      return FALSE;
+    else
+      return TRUE;
 }
 
-guint gwaei_2digithexstrtoint(char hex_char_digit_2, char hex_char_digit_1)
+//!
+//! @brief Converts a 2 digit hex number from a string to an integer
+//!
+//! @param the higher hex digit
+//! @param the lower hex digit
+//! @return Returns the int form of the hex number
+//!
+guint gw_util_2digithexstrtoint(char hex_char_digit_2, char hex_char_digit_1)
 {
     int digit1;
     if (hex_char_digit_1 <= '9')
@@ -105,7 +141,14 @@ guint gwaei_2digithexstrtoint(char hex_char_digit_2, char hex_char_digit_1)
 }
 
 
-gboolean gwaei_itohexstr(char *color_string, guint color_integer)
+//!
+//! @brief Creates a hex string out of an integer
+//!
+//! @param a pre-allocated string to write into
+//! @param an integer representing a color
+//! @return Always returns true
+//!
+gboolean gw_util_itohexstr (char *color_string, guint color_integer)
 {
     char *color_string_ptr = color_string;
     *color_string_ptr = '#';
@@ -134,17 +177,20 @@ gboolean gwaei_itohexstr(char *color_string, guint color_integer)
     return TRUE;
 }
 
-
-
-
-//Returns the string where the waei directory should be.  If it doesn't exist, it creates it
-char* get_waei_directory(char *buffer) 
+//!
+//! @brief Gets the location of the waei dictionary folder
+//!
+//! @param a pre-allocated string to write into
+//! @param an integer representing a color
+//! @return Always returns true
+//!
+char* gw_util_get_waei_directory(char *buffer) 
 {
-    //Get the path of the waei directory
-    strncpy(buffer, g_getenv("HOME"), FILENAME_MAX);
-    strcat(buffer, G_DIR_SEPARATOR_S);
-    strcat(buffer, ".waei");
-    strcat(buffer, G_DIR_SEPARATOR_S);
+    const char* home = g_get_home_dir ();
+    char* directory = g_build_filename (home, ".waei", NULL);
+    strcpy (buffer, directory);
+    strcat (buffer, G_DIR_SEPARATOR_S);
+    g_free (directory);
 
     //Make sure the ~/.waei directory exits, return NULL on error
     if(g_mkdir_with_parents (buffer, 0755) == -1)
@@ -154,59 +200,123 @@ char* get_waei_directory(char *buffer)
 }
 
 
-
-//Check if characters are romanji, katakana, hiragana, or kanji
-gboolean gwaei_all_chars_are_in_range ( char input[],
-                                        int  start_unic_boundary,
-                                        int  end_unic_boundary    )
+//!
+//! @brief Makes sure that all of the characters are in a specific range
+//!
+//! This function was made for the general purpose of quickly figuring out if
+//! a character string is all katakana, hiragana, or romaji.
+//!
+//! @param input The string to check
+//! @param start_unic_boundary the lower bound
+//! @param end_unic_boundary the upper bound
+//! @return Returns true if it is in the range
+//! @see gw_util_is_hiragana_str ()
+//! @see gw_util_is_katakana_str ()
+//! @see gw_util_is_kanji_str ()
+//! @see gw_util_is_romaji_str ()
+//!
+gboolean gw_util_all_chars_are_in_range (char input[],
+                                         int  start_unic_boundary,
+                                         int  end_unic_boundary   )
 {
-  //Setup
-  char *input_ptr;
-  input_ptr = input;
+    //Setup
+    char *input_ptr;
+    input_ptr = input;
 
-  gunichar unic;
-  unic = g_utf8_get_char(input_ptr);
-
-  gboolean is_in_boundary;
-  is_in_boundary = (unic >= start_unic_boundary && unic <= end_unic_boundary);
-
-  //Check
-  while (*input_ptr != '\0' && is_in_boundary)
-  {
-    input_ptr = g_utf8_next_char(input_ptr);
+    gunichar unic;
     unic = g_utf8_get_char(input_ptr);
+
+    gboolean is_in_boundary;
     is_in_boundary = (unic >= start_unic_boundary && unic <= end_unic_boundary);
-  }
 
-  //Return Results
-  return (input[0] != '\0' && *input_ptr == '\0');
+    //Check
+    while (*input_ptr != '\0' && is_in_boundary)
+    {
+      input_ptr = g_utf8_next_char(input_ptr);
+      unic = g_utf8_get_char(input_ptr);
+      is_in_boundary = (unic >= start_unic_boundary && unic <= end_unic_boundary);
+    }
+
+    //Return Results
+    return (input[0] != '\0' && *input_ptr == '\0');
 }
 
 
-//Convenience functions
-gboolean gwaei_util_is_hiragana_str(char input[])
+//!
+//! @brief Convenience function for seeing if a string is hiragana
+//!
+//! @param input The string to check
+//! @return Returns true if it is in the range
+//! @see gw_util_all_chars_are_in_range ()
+//! @see gw_util_is_katakana_str ()
+//! @see gw_util_is_kanji_str ()
+//! @see gw_util_is_romaji_str ()
+//!
+gboolean gw_util_is_hiragana_str (char input[])
 {
-    return gwaei_all_chars_are_in_range (input, L'ぁ', L'ん');
+    return gw_util_all_chars_are_in_range (input, L'ぁ', L'ん');
 }
 
-gboolean gwaei_util_is_katakana_str(char input[])
+
+//!
+//! @brief Convenience function for seeing if a string is katakana
+//!
+//! @param input The string to check
+//! @return Returns true if it is in the range
+//! @see gw_util_all_chars_are_in_range ()
+//! @see gw_util_is_hiragana_str ()
+//! @see gw_util_is_kanji_str ()
+//! @see gw_util_is_romaji_str ()
+//!
+gboolean gw_util_is_katakana_str (char input[])
 {
-    return gwaei_all_chars_are_in_range (input, L'ァ', L'ー');
+    return gw_util_all_chars_are_in_range (input, L'ァ', L'ー');
 }
 
-gboolean gwaei_util_is_kanji_str(char input[])
+
+//!
+//! @brief Convenience function for seeing if a string is kanji
+//!
+//! @param input The string to check
+//! @return Returns true if it is in the range
+//! @see gw_util_all_chars_are_in_range ()
+//! @see gw_util_is_hiragana_str ()
+//! @see gw_util_is_katakana_str ()
+//! @see gw_util_is_romaji_str ()
+//!
+gboolean gw_util_is_kanji_str (char input[])
 {
-    return gwaei_all_chars_are_in_range (input, L'ー', 0xFF00);
+    return gw_util_all_chars_are_in_range (input, L'ー', 0xFF00);
 }
 
-gboolean gwaei_util_is_romanji_str(char input[])
+
+//!
+//! @brief Convenience function for seeing if a string is romaji
+//!
+//! @param input The string to check
+//! @return Returns true if it is in the range
+//! @see gw_util_all_chars_are_in_range ()
+//! @see gw_util_is_hiragana_str ()
+//! @see gw_util_is_katakana_str ()
+//! @see gw_util_is_kanji_str ()
+//!
+gboolean gw_util_is_romaji_str (char input[])
 {
-    return gwaei_all_chars_are_in_range (input, L'A', L'z');
+    return gw_util_all_chars_are_in_range (input, L'A', L'z');
 }
 
 
-//Shift characters between hiragana and katakana
-void gwaei_shift_all_chars_in_str_by(char input[], int shift)
+//!
+//! @brief Shifts the characters in a specific direction
+//!
+//! This function is used for hiragana to katakana conversions and vice versa.
+//!
+//! @param input The string to shift
+//! @param shift How much to shift by
+//! @see gw_util_str_shift_hira_to_kata ()
+//! @see gw_util_str_shift_kata_to_hira ()
+//!
+void gw_util_shift_all_chars_in_str_by (char input[], int shift)
 {
     //Setup
     char *input_ptr;
@@ -239,28 +349,42 @@ void gwaei_shift_all_chars_in_str_by(char input[], int shift)
 }
 
 
-//
-//Convenience functions
-//
-
-
-gwaei_str_shift_hiragana_to_katakana(char input[])
+//!
+//! @brief Convenience function to shift hiragana to katakana
+//!
+//! @param input The string to shift
+//! @see gw_util_shift_all_chars_in_str_by ()
+//! @see gw_util_str_shift_kata_to_hira ()
+//!
+void gw_util_str_shift_hira_to_kata (char input[])
 {
-    gwaei_shift_all_chars_in_str_by(input, (L'ア' - L'あ'));
-}
-
-gwaei_str_shift_katakana_to_hiragana(char input[])
-{
-    gwaei_shift_all_chars_in_str_by(input, (L'あ' - L'ア'));
+    gw_util_shift_all_chars_in_str_by (input, (L'ア' - L'あ'));
 }
 
 
-//
-//Functions for managing romanji to hiragana conversions
-//
+//!
+//! @brief Convenience function to shift katakana to hiragana
+//!
+//! @param input The string to shift
+//! @see gw_util_shift_all_chars_in_str_by ()
+//! @see gw_util_str_shift_hira_to_kata ()
+//!
+void gw_util_str_shift_kata_to_hira (char input[])
+{
+    gw_util_shift_all_chars_in_str_by (input, (L'あ' - L'ア'));
+}
 
 
-char* gwaei_next_hiragana_char_from_romanji(char *input)
+//!
+//! @brief Gets the next hiragana equivalent char pointer in a string
+//!
+//! This function returns the hiragana equivalent and skips the romanji equivalent
+//! forward in the string.  This function is used for romaji->hiragana conversions.
+//!
+//! @param input The string to jump around
+//! @return where the next hiragana equivalent character would start
+//!
+char* gw_util_next_hira_char_from_roma (char *input)
 {
     char *input_ptr;
     input_ptr = input;
@@ -317,12 +441,26 @@ char* gwaei_next_hiragana_char_from_romanji(char *input)
       //Increment
       input_ptr++;
     }
+    if (*(input_ptr - 1) == 'n' &&
+            *(input_ptr) == 'n'
+       )
+      input_ptr++;
+
 
     return input_ptr;
 }
 
 
-char* gwaei_romanji_to_hiragana(char *input, char *output)
+//!
+//! @brief Converts a romaji string to hiragana.
+//!
+//! Attempts to convert a romaji string to hiragana.
+//!
+//! @param input The source romaji string
+//! @param input The string to write the hiragana equivalent to
+//! @return Returns null on error/end
+//!
+char* gw_util_roma_to_hira (char *input, char *output)
 {
     //Set up the input pointer
     char *input_ptr;
@@ -335,7 +473,7 @@ char* gwaei_romanji_to_hiragana(char *input, char *output)
     char buffer[] = "           ";
     char *buffer_ptr = buffer;
 
-    //Copy the next hiragana char written in romanji to the buffer
+    //Copy the next hiragana char written in romaji to the buffer
     while ( 
             (
               *input_ptr != '\0' &&
@@ -396,15 +534,15 @@ char* gwaei_romanji_to_hiragana(char *input, char *output)
        strcpy(output, "ん");
 
 
-    else if ( strlen(buffer_ptr) == 1 &&
-              buffer_ptr[0] != 'a'    &&
-              buffer_ptr[0] != 'i'    &&
-              buffer_ptr[0] != 'u'    &&
-              buffer_ptr[0] != 'e'    &&
-              buffer_ptr[0] != 'o'    &&
-              buffer_ptr[0] != '-'    &&
-              buffer_ptr[0] != 'y'    &&
-              input_ptr[1] != '\0'      )
+    else if (strlen(buffer_ptr) == 1 &&
+             buffer_ptr[0] != 'a'    &&
+             buffer_ptr[0] != 'i'    &&
+             buffer_ptr[0] != 'u'    &&
+             buffer_ptr[0] != 'e'    &&
+             buffer_ptr[0] != 'o'    &&
+             buffer_ptr[0] != '-'    &&
+             buffer_ptr[0] != 'y'    &&
+             input_ptr[1] != '\0'      )
        strcpy(output, "っ");
 
     else if (strcmp(buffer_ptr, "a") == 0)
@@ -444,7 +582,7 @@ char* gwaei_romanji_to_hiragana(char *input, char *output)
     else if (strcmp(buffer_ptr, "gu") == 0)
        strcpy(output, "ぐ");
     else if (strcmp(buffer_ptr, "ge") == 0)
-       strcpy(output, "け");
+       strcpy(output, "げ");
     else if (strcmp(buffer_ptr, "go") == 0)
        strcpy(output, "ご");
 
@@ -703,22 +841,35 @@ char* gwaei_romanji_to_hiragana(char *input, char *output)
 }
 
 
-gboolean is_japanese_ctype()
+//!
+//! @brief Checks for a Japanese ctype localization setting
+//!
+//! @return Returns true if it is a japanese ctype
+//!
+gboolean gw_util_is_japanese_ctype ()
 {
-    return ( setlocale(LC_CTYPE, NULL) != NULL &&
-             (
-               strcmp(setlocale(LC_CTYPE, NULL), "ja_JP.UTF8")  == 0 ||
-               strcmp(setlocale(LC_CTYPE, NULL), "ja_JP.UTF-8") == 0 ||
-               strcmp(setlocale(LC_CTYPE, NULL), "ja_JP.utf8")  == 0 ||
-               strcmp(setlocale(LC_CTYPE, NULL), "ja_JP.utf-8") == 0 ||
-               strcmp(setlocale(LC_CTYPE, NULL), "ja_JP")       == 0 ||
-               strcmp(setlocale(LC_CTYPE, NULL), "ja")          == 0 ||
-               strcmp(setlocale(LC_CTYPE, NULL), "japanese")    == 0
-             )
+    return (setlocale(LC_CTYPE, NULL) != NULL &&
+            (
+              strcmp(setlocale(LC_CTYPE, NULL), "ja_JP.UTF8")  == 0 ||
+              strcmp(setlocale(LC_CTYPE, NULL), "ja_JP.UTF-8") == 0 ||
+              strcmp(setlocale(LC_CTYPE, NULL), "ja_JP.utf8")  == 0 ||
+              strcmp(setlocale(LC_CTYPE, NULL), "ja_JP.utf-8") == 0 ||
+              strcmp(setlocale(LC_CTYPE, NULL), "ja_JP")       == 0 ||
+              strcmp(setlocale(LC_CTYPE, NULL), "ja")          == 0 ||
+              strcmp(setlocale(LC_CTYPE, NULL), "japanese")    == 0
+            )
            );
 }
 
-gboolean is_japanese_locale()
+//!
+//! @brief Checks for a Japanese local messages setting
+//!
+//! Basically this checks if the interface is supposed to come out to be
+//! Japanese in the program or not.
+//!
+//! @return Returns true if it is a japanese local
+//!
+gboolean gw_util_is_japanese_locale()
 {
     return ( setlocale(LC_MESSAGES, NULL) != NULL &&
              (
@@ -734,30 +885,41 @@ gboolean is_japanese_locale()
 }
 
 
-void gwaei_util_strncpy_fallback_from_key (char *value, char *key, int n)
+//!
+//! @brief A function that gets the default pref hardwired in.
+//!
+//! This function is made to take things like gconf out of the equation
+//! so that the program can run even if it doesn't have any preferences
+//! stored anywhere.
+//!
+//! @param value The pre-allocated string to copy the value to
+//! @param key the key of the wanted preference
+//! @param n the radix of the size of the value array
+//!
+void gw_util_strncpy_fallback_from_key (char *value, char *key, int n)
 {
-  if (strcmp (GCKEY_GWAEI_ENGLISH_SOURCE, key) == 0)
-    strncpy (value, GWAEI_ENGLISH_URI_FALLBACK, n);
-  else if (strcmp (GCKEY_GWAEI_KANJI_SOURCE, key) == 0)
-    strncpy (value, GWAEI_KANJI_URI_FALLBACK, n);
-  else if (strcmp (GCKEY_GWAEI_NAMES_SOURCE, key) == 0)
-    strncpy (value, GWAEI_NAMES_URI_FALLBACK, n);
-  else if (strcmp (GCKEY_GWAEI_RADICALS_SOURCE, key) == 0)
-    strncpy (value, GWAEI_RADICALS_URI_FALLBACK, n);
+    if (strcmp (GCKEY_GW_ENGLISH_SOURCE, key) == 0)
+      strncpy (value, GW_ENGLISH_URI_FALLBACK, n);
+    else if (strcmp (GCKEY_GW_KANJI_SOURCE, key) == 0)
+      strncpy (value, GW_KANJI_URI_FALLBACK, n);
+    else if (strcmp (GCKEY_GW_NAMES_SOURCE, key) == 0)
+      strncpy (value, GW_NAMES_URI_FALLBACK, n);
+    else if (strcmp (GCKEY_GW_RADICALS_SOURCE, key) == 0)
+      strncpy (value, GW_RADICALS_URI_FALLBACK, n);
 
-  else if (strcmp (GCKEY_GWAEI_MATCH_FG, key) == 0)
-    strncpy (value, GWAEI_MATCH_FG_FALLBACK, n);
-  else if (strcmp (GCKEY_GWAEI_MATCH_BG, key) == 0)
-    strncpy (value, GWAEI_MATCH_BG_FALLBACK, n);
-  else if (strcmp (GCKEY_GWAEI_HEADER_FG, key) == 0)
-    strncpy (value, GWAEI_HEADER_FG_FALLBACK, n);
-  else if (strcmp (GCKEY_GWAEI_HEADER_BG, key) == 0)
-    strncpy (value, GWAEI_HEADER_BG_FALLBACK, n);
-  else if (strcmp (GCKEY_GWAEI_COMMENT_FG, key) == 0)
-    strncpy (value, GWAEI_COMMENT_FG_FALLBACK, n);
+    else if (strcmp (GCKEY_GW_MATCH_FG, key) == 0)
+      strncpy (value, GW_MATCH_FG_FALLBACK, n);
+    else if (strcmp (GCKEY_GW_MATCH_BG, key) == 0)
+      strncpy (value, GW_MATCH_BG_FALLBACK, n);
+    else if (strcmp (GCKEY_GW_HEADER_FG, key) == 0)
+      strncpy (value, GW_HEADER_FG_FALLBACK, n);
+    else if (strcmp (GCKEY_GW_HEADER_BG, key) == 0)
+      strncpy (value, GW_HEADER_BG_FALLBACK, n);
+    else if (strcmp (GCKEY_GW_COMMENT_FG, key) == 0)
+      strncpy (value, GW_COMMENT_FG_FALLBACK, n);
 
-  else
-    strncpy (value, "", n);
+    else
+      strncpy (value, "", n);
 }
 
 
